# -*- coding: utf-8 -*-
# .

__project_name__ = 'Defter'
__author__ = 'E.Y.'
__date__ = '14/03/2024'

from datetime import timedelta


########################################################################
class SrtAyikla:

    # ---------------------------------------------------------------------
    def __init__(self, dosya_adresi):

        self.dosya_adresi = dosya_adresi

        # self.blokSozluk = {}

        self.satirlar = []

        self.baslamaZamanlari = []
        self.bitisZamanlari = []
        self.yazilar = []

    # ---------------------------------------------------------------------
    def oku(self):
        with open(self.dosya_adresi, "r", encoding="utf-8") as f:
            self.satirlar = f.readlines()

    # # ---------------------------------------------------------------------
    # def sozluge_ayikla(self):
    #     self.oku()
    #     su_an_ki_sira = 0
    #
    #     i = 0
    #     for i in range (len(self.satirlar)):
    #         satir = self.satirlar[i].strip()
    #         if satir:
    #             if satir.isdigit():
    #                 satir_int = int(satir)
    #                 sonraki_sira_no_indexi = self.sonraki_sira_no_ve_indexi(i, satir_int)
    #                 self.boslari_temizle_blok_sozluge_ekle(satir_int,i+1,sonraki_sira_no_indexi)
    #                 i=sonraki_sira_no_indexi
    #                 continue
    #         i += 1
    #
    #     # print(self.blokSozluk)
    #     # for k,v in self.blokSozluk.items():
    #     #     print(k)
    #     #     print(v)

    # # ---------------------------------------------------------------------
    # def boslari_temizle_blok_sozluge_ekle(self, satir_int, z, sonraki_sira_no_indexi):
    #     # self.blokSozluk[satir_int] = (self.satirlar[i:sonraki_sira_no_indexi])
    #     blok_liste = []
    #     yazi = ""
    #     for y in range(z, sonraki_sira_no_indexi):
    #         satir = self.satirlar[y].strip()
    #         if satir:
    #             if not blok_liste:
    #                 bas,son =self.ilk_son_zaman_cevir(satir)
    #                 blok_liste.extend((bas,son))
    #             else:
    #                 yazi =f"{yazi} {satir}"
    #         y += 1
    #
    #     if yazi:
    #         blok_liste.append(yazi)
    #         self.blokSozluk[satir_int] = blok_liste

    # ---------------------------------------------------------------------
    def sonraki_altyazinin_sira_no_indexini_bul(self, su_an_ki_sira, satir_int):
        satir_int_arti_bir = satir_int + 1
        for i, satir in enumerate(self.satirlar[su_an_ki_sira:]):
            satir = satir.strip()
            if satir:
                if satir.isdigit():
                    satir_int = int(satir)
                    # TODO: burda mesela yazi satiri "10" olsa ve biz 9. altyaziyi ayikliyor olsak
                    # problem olabilir, dusuk de olsa ihtimal var.
                    # sonraki bir kaç satira da bakmak lazim
                    if satir_int == satir_int_arti_bir:
                        # mesela belki,
                        # if "-->" in self.satirlar[su_an_ki_sira + i + 1]:
                        #     break
                        break
        return i + su_an_ki_sira

    # ---------------------------------------------------------------------
    def listelere_ayikla(self):

        self.oku()

        i = 0
        while i < len(self.satirlar):
            satir = self.satirlar[i].strip()
            if satir:
                if satir.isdigit():
                    satir_int = int(satir)
                    sonraki_sira_no_indexi = self.sonraki_altyazinin_sira_no_indexini_bul(i, satir_int)
                    self.boslari_temizle_listelere_ekle(i + 1, sonraki_sira_no_indexi)
                    # i ve sonraki_sira_no_indexi arasindaki satirlar islendi listelere eklendi
                    # dolayisi ile i= sonraki_sira_no_indexi deyip, ordan devam ediyoruz
                    i = sonraki_sira_no_indexi
                    continue
            i += 1

        return self.baslamaZamanlari, self.yazilar, self.bitisZamanlari

    # ---------------------------------------------------------------------
    def boslari_temizle_listelere_ekle(self, blok_icerik_baslangic, sonraki_sira_no_indexi):
        # self.blokSozluk[satir_int] = (self.satirlar[i:sonraki_sira_no_indexi])
        # blok_liste = []
        yazi = ""
        bas = son = None
        for i in range(blok_icerik_baslangic, sonraki_sira_no_indexi):
            satir = self.satirlar[i].strip()
            if satir:
                if bas is None:
                    bas, son = self.ilk_son_zaman_cevir(satir)
                else:
                    yazi = f"{yazi} {satir}"
            i += 1

        if yazi:
            try:
                if yazi in self.yazilar[-1]:
                    self.yazilar[-1] = yazi
                    # self.baslamaZamanlari[-1] = bas
                    self.bitisZamanlari[-1] = son
                else:
                    self.yazilar.append(yazi)
                    self.baslamaZamanlari.append(bas)
                    self.bitisZamanlari.append(son)
            except IndexError:
                self.yazilar.append(yazi)
                self.baslamaZamanlari.append(bas)
                self.bitisZamanlari.append(son)

    # ---------------------------------------------------------------------
    def ilk_son_zaman_cevir(self, zaman):

        # :)
        zaman = zaman.replace(".", " ").replace(",", " ").replace(":", " ").replace("-->", " ")

        basH, basM, basS, basMs, sonH, sonM, sonS, sonMs = zaman.split()

        bas_ms = timedelta(hours=int(basH),
                           minutes=int(basM),
                           seconds=int(basS),
                           milliseconds=int(basMs)) // timedelta(milliseconds=1)

        son_ms = timedelta(hours=int(sonH),
                           minutes=int(sonM),
                           seconds=int(sonS),
                           milliseconds=int(sonMs)) // timedelta(milliseconds=1)

        return bas_ms, son_ms


# ---------------------------------------------------------------------
def calistir():
    # pass
    altyaziAdres = "altyazi2.srt"
    s = SrtAyikla(altyaziAdres)
    # bir_sey_dondurmuyor = s.sozluge_ayikla()
    basL, yaziL, sonL = s.listelere_ayikla()
    print(basL)
    print(yaziL)
    print(sonL)


# ---------------------------------------------------------------------
if __name__ == "__main__":
    calistir()



# -*- coding: utf-8 -*-
# .

__project_name__ = 'defter'
__date__ = '06/03/24'
__author__ = 'E. Y.'

import os
import time
from PySide6.QtCore import Qt, Signal
from PySide6.QtGui import QColor, QBrush
from PySide6.QtWidgets import (QHBoxLayout, QWidget, QVBoxLayout, QTableWidget, QTableWidgetItem, QFileDialog, QAbstractItemView, QComboBox,
                               QCheckBox)

from .. import shared
from ...canta.pushButton import PushButton
from ...canta.lineEdit import AraLineEdit
from ..yw.yuzenWidget import YuzenWidget


#######################################################################
class AltyaziYW(YuzenWidget):
    log = Signal(str)
    sonKlasorVideolarGuncelle = Signal()
    sahneAktifNesneBilgisiGuncelle = Signal()

    # ---------------------------------------------------------------------
    def __init__(self, parent=None):
        super(AltyaziYW, self).__init__(kapatilabilir_mi=False, parent=parent)

        temelW = QWidget(self)
        temelW.setContentsMargins(0, 0, 0, 0)
        anaLay = QVBoxLayout(temelW)
        anaLay.setSizeConstraint(QVBoxLayout.SizeConstraint.SetMinAndMaxSize)
        # anaLay.setContentsMargins(3, 5, 3, 3)
        # anaLay.setContentsMargins(1, 3, 1, 1)
        anaLay.setContentsMargins(1, 3, 1, 3)
        anaLay.setSpacing(3)

        btnRenk = QColor(200, 200, 200)

        self.otoGuncelleCBox = QCheckBox(self)
        self.otoGuncelleCBox.setMaximumWidth(15)
        self.otoGuncelleCBox.setToolTip(self.tr("Auto refresh subtitles when a video item is selected"))
        self.otoGuncelleCBox.setStatusTip(self.tr("Auto refresh subtitles when a video item is selected"))
        self.otoGuncelleCBox.setChecked(True)

        self.yukleBtn = PushButton(self.tr('Load'), yukseklik=16, renkArkaplan=btnRenk, parent=temelW)
        self.yukleBtn.setFixedWidth(35)
        self.yukleBtn.setToolTip(self.tr("Load Subtitle"))
        self.yukleBtn.clicked.connect(self.altyazi_yukle)
        # self.yukleBtn.setDisabled(True)

        self.araLineEdit = AraLineEdit(self)
        # self.araLineEdit.setMinimumWidth(250)
        self.araLineEdit.setPlaceholderText(self.tr("Search..."))
        self.araLineEdit.returnVeyaEnterBasildi.connect(self.act_ara)
        # self.araLineEdit.shiftEnterVeyaReturnBasildi.connect(self.act_ara)

        self.araLineEdit.temizleBtnClicked.connect(self.act_aramayi_temizle)
        self.araLineEdit.aramaMetniDegisti.connect(lambda: self.act_aramayi_temizle(yeniden_ara=True))

        self.zamanAcKapaBtn = PushButton(self.tr('0:0'), yukseklik=16, renkArkaplan=btnRenk, parent=temelW)
        self.zamanAcKapaBtn.setFixedWidth(35)
        self.zamanAcKapaBtn.setToolTip(self.tr("Timecode On / Off"))
        self.zamanAcKapaBtn.clicked.connect(self.zaman_ac_kapa)
        # self.zamanAcKapaBtn.setDisabled(True)

        self.oncekiAltyaziBtn = PushButton(self.tr('<'), yukseklik=16, renkArkaplan=btnRenk, parent=temelW)
        self.oncekiAltyaziBtn.setFixedWidth(35)
        self.oncekiAltyaziBtn.setToolTip(self.tr("Previous Subtitle"))
        self.oncekiAltyaziBtn.clicked.connect(self.onceki_altyaziya_git)
        # self.oncekiAltyaziBtn.setDisabled(True)

        self.sonrakiAltyaziBtn = PushButton(self.tr('>'), yukseklik=16, renkArkaplan=btnRenk, parent=temelW)
        self.sonrakiAltyaziBtn.setFixedWidth(35)
        self.sonrakiAltyaziBtn.setToolTip(self.tr("Next subtitle"))
        self.sonrakiAltyaziBtn.clicked.connect(self.sonraki_altyaziya_git)
        # self.sonrakiAltyaziBtn.setDisabled(True)

        self.oynatDurdurBtn = PushButton(self.tr('//>'), yukseklik=16, renkArkaplan=btnRenk, parent=temelW)
        self.oynatDurdurBtn.setFixedWidth(35)
        self.oynatDurdurBtn.setToolTip(self.tr("Play / Pause"))
        self.oynatDurdurBtn.clicked.connect(self.oynat_durdur)
        # self.oynatDurdurBtn.setDisabled(True)

        self.dongudeOynatBtn = PushButton(self.tr('[o-o]'), yukseklik=16, renkArkaplan=btnRenk, parent=temelW)
        self.dongudeOynatBtn.setFixedWidth(35)
        self.dongudeOynatBtn.setToolTip(self.tr("Play selected subtitles in loop"))
        self.dongudeOynatBtn.clicked.connect(self.oynat_dongude)
        self.dongudeOynatBtn.setCheckable(True)
        # self.dongudeOynatBtn.setDisabled(True)

        self.altyaziCBox = QComboBox(temelW)
        self.altyaziCBox.setEditable(False)
        # self.altyaziCBox.currentIndexChanged.connect(self.altyazi_degistir_cbox)
        self.altyaziCBox.activated.connect(self.altyazi_degistir_cbox)

        layYukle = QHBoxLayout()
        # layYukle.addWidget(lblOtoGuncelle)
        layYukle.addWidget(self.otoGuncelleCBox)
        layYukle.addWidget(self.yukleBtn)
        # layYukle.addStretch()
        layYukle.addWidget(self.altyaziCBox)
        layOynatmaKontrol = QHBoxLayout()
        layOynatmaKontrol.addWidget(self.zamanAcKapaBtn)
        layOynatmaKontrol.addWidget(self.oncekiAltyaziBtn)
        layOynatmaKontrol.addWidget(self.sonrakiAltyaziBtn)
        layOynatmaKontrol.addWidget(self.oynatDurdurBtn)
        layOynatmaKontrol.addWidget(self.dongudeOynatBtn)
        layOynatmaKontrol.addStretch()
        layOynatmaKontrol.addSpacing(10)

        self.dongude = False

        self.altyaziTW = QTableWidget(temelW)
        self.altyaziTW.setObjectName("aTW")
        self.altyaziTW.setDragEnabled(True)
        self.altyaziTW.setColumnCount(2)
        self.altyaziTW.horizontalHeader().hide()
        self.altyaziTW.verticalHeader().hide()
        self.altyaziTW.setEditTriggers(QAbstractItemView.EditTrigger.NoEditTriggers)
        self.altyaziTW.setWordWrap(True)
        self.altyaziTW.setSelectionBehavior(QAbstractItemView.SelectionBehavior.SelectRows)
        # self.altyaziTW.setSelectionMode(QAbstractItemView.SelectionMode.MultiSelection)

        self.altyaziTW.itemDoubleClicked.connect(self.tiklanan_altyazi_zamanina_git)
        # self.altyaziTW.currentItemChanged.connect(self.loop)
        self.altyaziTW.itemSelectionChanged.connect(self.loop)
        # self.altyaziTW.itemSelectionChanged.connect(self.act_stylePresetsListWidget_secim_degisti)
        # self.altyaziTW.setContextMenuPolicy(Qt.ContextMenuPolicy.CustomContextMenu)
        # self.altyaziTW.customContextMenuRequested.connect(self.act_stiller_sag_menu_goster)

        self.dongudekiSatirBrush = QBrush(QColor(220, 225, 235))
        self.orjinalBrush = QTableWidgetItem().background()
        self.donguSecilenSatirIndexleri = []

        qss_stil = """
                    QTableViewt{outline: none;}
                    QTableWidget::item{padding: 3px 0;}
                    QTableWidget::item:selected {color: #ffffff; background: #81aac5; border:none;}
                    """

        self.altyaziTW.setStyleSheet(qss_stil)
        # self.altyaziTW.setAttribute(Qt.WA_MacShowFocusRect, 0)

        anaLay.addLayout(layYukle)
        anaLay.addWidget(self.araLineEdit)
        anaLay.addWidget(self.altyaziTW)
        anaLay.addLayout(layOynatmaKontrol)
        # anaLay.addStretch()

        self.ekleWidget(temelW)

        self.setMinimumWidth(250)
        self.setMinimumHeight(170)

        self.sonKlasorVideolar = ""
        self.sahneAktifNesne = None

    # ---------------------------------------------------------------------
    def sonKlasorVideolar_guncelle(self, klasor):
        self.sonKlasorVideolar = klasor

    # ---------------------------------------------------------------------
    def sahneAktifNesneBilgisi_guncelle(self, nesne):
        # self.sahneAktifNesne = weakref.ref(nesne)
        self.sahneAktifNesne = nesne

    # ---------------------------------------------------------------------
    def zaman_ac_kapa(self):
        self.altyaziTW.setColumnHidden(0, not self.altyaziTW.isColumnHidden(0))

    # ---------------------------------------------------------------------
    def act_aramayi_temizle(self, yeniden_ara=False):
        # TODO: bu undoRedoSiniflarda 6 yerde var
        # print("temizlendi")
        for satir in range(self.altyaziTW.rowCount()):
            self.altyaziTW.setRowHidden(satir, False)
            # if self.isVisible():
            if yeniden_ara:
                self.act_ara()

    # ---------------------------------------------------------------------
    def act_ara(self):
        for satir in range(self.altyaziTW.rowCount()):
            nesne = self.altyaziTW.item(satir, 1)
            if not self.araLineEdit.text() in nesne.text():
                self.altyaziTW.setRowHidden(satir, True)

    # ---------------------------------------------------------------------
    def oynat_durdur(self):
        self.dongude = False
        if self.donguSecilenSatirIndexleri:
            for satirIdx in self.donguSecilenSatirIndexleri:
                self.altyaziTW.item(satirIdx.row(), 0).setBackground(self.orjinalBrush)
                self.altyaziTW.item(satirIdx.row(), 1).setBackground(self.orjinalBrush)
            self.donguSecilenSatirIndexleri = []

        videoNesne = self.videoNesnesi()
        if videoNesne:
            if videoNesne.player.playbackState() == videoNesne.player.PlaybackState.PlayingState:
                videoNesne.player.pause()
            else:
                videoNesne.player.play()

    # ---------------------------------------------------------------------
    def sonraki_altyaziya_git(self):
        satir = self.altyaziTW.currentRow()
        if not satir == -1:
            self.altyaziTW.selectRow(satir + 1)
            self.tiklanan_altyazi_zamanina_git(self.altyaziTW.currentItem())

    # ---------------------------------------------------------------------
    def onceki_altyaziya_git(self):
        satir = self.altyaziTW.currentRow()
        if not satir == -1:
            self.altyaziTW.selectRow(satir - 1)
            self.tiklanan_altyazi_zamanina_git(self.altyaziTW.currentItem())

    # ---------------------------------------------------------------------
    def oynat_dongude(self):
        self.dongude = True
        self.donguSecilenSatirIndexleri = self.altyaziTW.selectionModel().selectedRows()
        if self.donguSecilenSatirIndexleri:

            for satirIdx in self.donguSecilenSatirIndexleri:
                self.altyaziTW.item(satirIdx.row(), 0).setBackground(self.dongudekiSatirBrush)
                self.altyaziTW.item(satirIdx.row(), 1).setBackground(self.dongudekiSatirBrush)

            self.baslaSatir = self.donguSecilenSatirIndexleri[0].row()
            self.bitirSatir = self.donguSecilenSatirIndexleri[-1].row() + 1
            self.tiklanan_altyazi_zamanina_git(self.altyaziTW.item(self.baslaSatir, 0))
            self.altyaziTW.selectRow(self.baslaSatir)

            videoNesne = self.videoNesnesi()
            if videoNesne:
                if not videoNesne.player.playbackState() == videoNesne.player.PlaybackState.PlayingState:
                    videoNesne.player.play()

    # ---------------------------------------------------------------------
    # def loop(self, item, onceki):
    def loop(self):
        if self.dongude:
            # print(self.altyaziTW.currentRow(), self.bitirSatir)
            if self.altyaziTW.currentRow() == self.bitirSatir:
                self.tiklanan_altyazi_zamanina_git(self.altyaziTW.item(self.baslaSatir, 0))
                self.altyaziTW.selectRow(self.baslaSatir)

    # ---------------------------------------------------------------------
    def videoNesnesi(self):
        self.sahneAktifNesneBilgisiGuncelle.emit()
        if self.sahneAktifNesne:
            if self.sahneAktifNesne.type() == shared.VIDEO_ITEM_TYPE:
                return self.sahneAktifNesne
        return None

    # ---------------------------------------------------------------------
    def satir_sec(self, satir):
        # TODO: aynı zamanda videoyu da sec, secili degilse
        self.altyaziTW.selectRow(satir)

    # ---------------------------------------------------------------------
    def tiklanan_altyazi_zamanina_git(self, nesne):
        # self.dongude = False
        videoNesne = self.videoNesnesi()
        if videoNesne:
            if videoNesne.altyazilarSozluk[videoNesne.simdikiAltyazi][0]:
                videoNesne.altyazi_sira_bul(videoNesne.player.position())
            # print(nesne.data(Qt.ItemDataRole.UserRole))
            # print(videoNesne.player.position())
            videoNesne.player_set_position(nesne.data(Qt.ItemDataRole.UserRole))

    # ---------------------------------------------------------------------
    def tasinanSatirlariYaziyaCevir(self):
        secilenIndexler = self.altyaziTW.selectionModel().selectedRows()
        yazi = ""
        for satirIdx in secilenIndexler:
            yazi = f"{yazi}\n{self.altyaziTW.item(satirIdx.row(), 1).text()}"
        return yazi

    # ---------------------------------------------------------------------
    def altyazi_degistir_cbox(self, idx):
        # print(self.altyaziCBox.itemText(idx))
        # print(self.altyaziCBox.itemData(idx))
        videoNesne = self.videoNesnesi()
        if not videoNesne.simdikiAltyazi == self.altyaziCBox.currentData():
            videoNesne.simdikiAltyazi = self.altyaziCBox.currentData()
            self.altyazilari_guncelle()

    # ---------------------------------------------------------------------
    def altyazi_yukle(self):
        videoNesne = self.videoNesnesi()
        if videoNesne:

            self.sonKlasorVideolarGuncelle.emit()
            filtre = self.tr("srt Files (*.srt);;All Files(*)")
            # filtre = self.tr("srt Files (*.srt);;webvtt Files (*.vtt);;All Files(*)")
            fn = QFileDialog.getOpenFileNames(self,
                                              self.tr("Open Subtitle Files..."),
                                              self.sonKlasorVideolar,
                                              filtre
                                              )
            # if fn:
            if fn[0]:
                altyaziAdresleri = fn[0]
                # altyaziAdres = altyaziAdresleri[0]
                t1 = time.time()
                videoNesne.altyazi_hesapla(altyaziAdresleri)
                self.log.emit(self.tr(f'Subtitles succesfully loaded!  -  {time.time() - t1:.2f} s'))
                self.sonKlasorVideolar = os.path.dirname(altyaziAdresleri[0])
        else:
            self.log.emit(self.tr("Please select a video item"))

    # ---------------------------------------------------------------------
    def altyazilari_guncelle(self):
        videoNesne = self.videoNesnesi()
        if videoNesne:
            self.altyaziCBox.clear()
            for altyaziAdres in videoNesne.altyazilarSozluk:
                self.altyaziCBox.addItem(os.path.basename(altyaziAdres), altyaziAdres)

            self.altyaziTW.clear()
            self.altyaziTW.setRowCount(len(videoNesne.altyazilarSozluk[videoNesne.simdikiAltyazi][0]))
            i = 0
            # altYaziBaslamaZamanlari , altYaziBitisZamanlari
            for pos_ms, altyazi in zip(videoNesne.altyazilarSozluk[videoNesne.simdikiAltyazi][0],
                                       videoNesne.altyazilarSozluk[videoNesne.simdikiAltyazi][1]):
                dk, sn = divmod((pos_ms // 1000), 60)
                saat, dk = divmod(dk, 60)
                nesne = QTableWidgetItem(f"{saat:02}:{dk:02}:{sn:02}")
                nesne.setData(Qt.ItemDataRole.UserRole, pos_ms)
                self.altyaziTW.setItem(i, 0, nesne)
                nesne = QTableWidgetItem(altyazi)
                nesne.setData(Qt.ItemDataRole.UserRole, pos_ms)
                self.altyaziTW.setItem(i, 1, nesne)
                i += 1

        self.altyaziTW.resizeColumnsToContents()
        self.altyaziTW.resizeRowsToContents()
        self.donguSecilenSatirIndexleri = []



# -*- coding: utf-8 -*-
# .

__project_name__ = 'Defter'
__author__ = 'Erdinç Yılmaz'
__date__ = '3/28/16'

from PySide6.QtCore import Qt, Signal, QSizeF, QRectF, QPointF, Slot, QUrl
from PySide6.QtGui import QBrush, QPen, QColor, QPainterPath, QFontMetricsF, QTextCursor
from PySide6.QtWidgets import QGraphicsTextItem, QStyle
from .. import shared


########################################################################
class Text(QGraphicsTextItem):
    Type = shared.TEXT_ITEM_TYPE

    # textItemFocusedOut = Signal(QGraphicsTextItem)
    textItemFocusedOut = Signal(object)

    # textItemSelectedChanged = Signal(QGraphicsTextItem)

    # ---------------------------------------------------------------------
    def __init__(self, cursorPos, yaziRengi, arkaPlanRengi, pen, font, rect=None, text=None, parent=None):
        super(Text, self).__init__(text, parent)

        self.aracTipi = "yaziAraci"

        self._kim = shared.kim(kac_basamak=16)

        self.setAcceptHoverEvents(True)

        self.doc = self.document()
        # self.doc.cursorPositionChanged.connect(self.doc_layout_changed)
        self.docLayout = self.doc.documentLayout()
        # self.docLayout.documentSizeChanged.connect(self.doc_layout_changed)

        # self.doc.setUndoRedoEnabled(True)

        self.doc.undoCommandAdded.connect(self.act_undo_eklendi)

        self.setFont(font)
        self.secili_nesne_kalem_kalinligi = 0

        self.isTextOverflowed = False
        self._resizing = False

        self.handleSize = 10
        self.resizeHandleSize = QSizeF(self.handleSize, self.handleSize)
        self.boundingRectTasmaDegeri = self.handleSize / 2

        if not rect:
            # self._rect = QRectF()
            self._rect = super(Text, self).boundingRect()
            # self._rect = QRectF(QPointF(0,0), self.doc.size())
            # self._rect = self.boundingRect()
        else:
            # rect.moveTo(0, 0)  # gerek yok cunku zaten 0,0 liyoruz scene den gondeririken.
            self._rect = rect
            self.setTextWidth(rect.size().width())

        self._boundingRect = self._rect.adjusted(-self.boundingRectTasmaDegeri, -self.boundingRectTasmaDegeri,
                                                 self.boundingRectTasmaDegeri, self.boundingRectTasmaDegeri)

        self.docLayout.documentSizeChanged.connect(self.doc_layout_changed)

        # self.setPos(cursorPos + QPointF(1, 1))
        self.setPos(cursorPos)

        self._eskiRectBeforeResize = None
        self._eskiPosBeforeResize = None

        self.update_resize_handles()

        # self.setCacheMode(Text.DeviceCoordinateCache)
        # self.setCacheMode(Text.ItemCoordinateCache)
        self.setCacheMode(Text.CacheMode.NoCache)
        self.setFlags(QGraphicsTextItem.GraphicsItemFlag.ItemIsSelectable |
                      QGraphicsTextItem.GraphicsItemFlag.ItemIsMovable |
                      QGraphicsTextItem.GraphicsItemFlag.ItemIsFocusable)

        # self.setTextInteractionFlags(Qt.TextEditable)
        self.setTextInteractionFlags(Qt.TextInteractionFlag.TextEditorInteraction)

        self.cursorPos = cursorPos

        self.hiza = Qt.AlignmentFlag.AlignLeft

        # fmt = QTextBlockFormat()
        # fmt.setAlignment(Qt.AlignCenter)
        cursor = self.textCursor()
        cursor.select(QTextCursor.SelectionType.Document)
        # cursor.mergeBlockFormat(fmt)
        # cursor.clearSelection()
        self.setTextCursor(cursor)

        self._brush = QBrush()
        # we dont use QPen in this text item.
        # self.cizgiKalinligi = 0
        # self.cizgiTipi = Qt.SolidLine
        # self.cizgiUcuTipi = Qt.RoundCap
        # self.cizgiBirlesimTipi = Qt.RoundJoin

        self.activeItemLineColor = shared.activeItemLineColor
        # self.setDefaultTextColor(cizgiRengi)
        self._pen = pen
        # self.cizgiRengi = QColor(Qt.black)
        self.cizgiRengi = pen.color()  # TODO: hemen asagidaki methodta yine set ediyor
        self.setCizgiRengi(self.cizgiRengi)  # also sets defaultTextColor
        self.yaziRengi = yaziRengi
        self.setYaziRengi(yaziRengi)
        self.setArkaPlanRengi(arkaPlanRengi)  # also sets self._brush

        self.cosmeticSelect = False
        self.isActiveItem = False
        self._isPinned = False
        self.ustGrup = None
        self._command = {}

        self.isPlainText = True
        self.oklar_dxdy_nokta = {}

        # self.zenginYaziBelirteci = QRect(0, 0, 10, 10)

        # self.cursor_eski_anchor = 0
        # self.cursor_eski_pozisyon = 0
        # self.doc.cursorPositionChanged.connect(self.cursor_pos_degisti)

    # # ---------------------------------------------------------------------
    # def cursor_pos_degisti(self, cursor):
    #     self.cursor_eski_pozisyon = cursor.position()
    #     self.cursor_eski_anchor = cursor.anchor()

    # ---------------------------------------------------------------------
    def type(self):
        # Enable the use of qgraphicsitem_cast with this item.
        return Text.Type

    # ---------------------------------------------------------------------
    @property
    def isPinned(self):
        return self._isPinned

    # ---------------------------------------------------------------------
    @isPinned.setter
    def isPinned(self, value):
        if value:
            self.setFlags(QGraphicsTextItem.GraphicsItemFlag.ItemIsSelectable
                          # | QGraphicsTextItem.GraphicsItemFlag.ItemIsMovable
                          #  | QGraphicsTextItem.GraphicsItemFlag.ItemIsFocusable
                          )

        else:
            self.setFlags(QGraphicsTextItem.GraphicsItemFlag.ItemIsSelectable
                          | QGraphicsTextItem.GraphicsItemFlag.ItemIsMovable
                          | QGraphicsTextItem.GraphicsItemFlag.ItemIsFocusable)
        self._isPinned = value

    # ---------------------------------------------------------------------
    def get_properties_for_save_binary(self):
        properties = {"type": self.type(),
                      "kim": self._kim,
                      "rect": self._rect,
                      "pos": self.pos(),
                      "rotation": self.rotation(),
                      "zValue": self.zValue(),
                      "pen": self._pen,
                      "font": self.font(),
                      "yaziRengi": self.yaziRengi,
                      "arkaPlanRengi": self.arkaPlanRengi,
                      "isPlainText": self.isPlainText,
                      "yaziHiza": int(self.doc.defaultTextOption().alignment()),
                      "isPinned": self.isPinned,
                      "command": self._command,
                      }
        if self.isPlainText:
            properties["text"] = self.toPlainText()
        else:
            properties["html"] = self.toHtml()
        return properties

    # ---------------------------------------------------------------------
    @Slot()
    def act_undo_eklendi(self):
        self.scene().undoRedo.undoRedoBaglantisiYaziNesnesiDocuna(self.scene().undoStack,
                                                                  self.scene().tr("change text"),
                                                                  self.doc)

    # ---------------------------------------------------------------------
    def ok_ekle(self, ok, scenepPos, okunHangiNoktasi):

        # self.oklar_dxdy_nokta.append((ok, self.mapFromScene(scenePos)))
        sceneCenter = self.sceneCenter()
        dx = sceneCenter.x() - scenepPos.x()
        dy = sceneCenter.y() - scenepPos.y()

        # self.oklar_dxdy_nokta[ok._kim] = (dx, dy, nokta)
        self.oklar_dxdy_nokta[ok] = (dx, dy, okunHangiNoktasi)

        ok.baglanmis_nesneler[self._kim] = okunHangiNoktasi

    # ---------------------------------------------------------------------
    def ok_sil(self, ok):
        del self.oklar_dxdy_nokta[ok]
        del ok.baglanmis_nesneler[self._kim]

    # ---------------------------------------------------------------------
    def ok_guncelle(self):
        if self.oklar_dxdy_nokta:
            sceneCenter = self.sceneCenter()
            scx = sceneCenter.x()
            scy = sceneCenter.y()
            for ok, dxdy_nokta in self.oklar_dxdy_nokta.items():
                if dxdy_nokta[2] == 1:
                    ok.temp_prepend(QPointF(scx - dxdy_nokta[0], scy - dxdy_nokta[1]))
                elif dxdy_nokta[2] == 2:
                    ok.temp_append(QPointF(scx - dxdy_nokta[0], scy - dxdy_nokta[1]))
                if ok._text:
                    ok.update_painter_text_rect()

    # ---------------------------------------------------------------------
    def varsaEnUsttekiGrubuGetir(self):
        parentItem = self.parentItem()
        while parentItem:
            if parentItem.type() == shared.GROUP_ITEM_TYPE:
                return parentItem
            parentItem = parentItem.parentItem()
        return None

    # ---------------------------------------------------------------------
    def set_document_url(self, url):

        # # ifdef Q_WS_WIN
        # document()->setMetaInformation(QTextDocument::DocumentUrl, imagePath + "/" );
        # # else
        # document()->setMetaInformation(QTextDocument::DocumentUrl, "file:" + imagePath + "/");
        # # endif

        # self.doc.setMetaInformation(self.doc.DocumentUrl, "file:{}/images/".format(url))

        # http://doc.qt.io/qt-5/qtextdocument.html#baseUrl-prop

        # TODO: bu aslinda boyle degil, url sonda / ile gelmiyor
        # url already comes with a "/" suffix so we use double slash "file://{url}"
        # self.doc.setBaseUrl(QUrl("file://{}/images-html/".format(url)))
        self.doc.setBaseUrl(QUrl("file://{}/".format(url)))
        # self.doc.setBaseUrl(QUrl(url))
        # print(self.doc.metaInformation(self.doc.DocumentUrl))

    # ---------------------------------------------------------------------
    def get_document_url(self):
        return self.doc.baseUrl()

    # ---------------------------------------------------------------------
    @Slot(QSizeF)
    def doc_layout_changed(self, size):
        # self._rect.setSize(self.doc.size())
        if size.width() > self._rect.size().width() or size.height() > self._rect.size().height():
            self._rect.setSize(size)
            self._boundingRect = self._rect.adjusted(-self.boundingRectTasmaDegeri,
                                                     -self.boundingRectTasmaDegeri,
                                                     self.boundingRectTasmaDegeri,
                                                     self.boundingRectTasmaDegeri)

            self.update_resize_handles()

    # ---------------------------------------------------------------------
    def update_resize_handles(self):
        self.prepareGeometryChange()

        if self.scene():
            resizeHandleSize = self.resizeHandleSize / self.scene().views()[0].transform().m11()
        else:
            resizeHandleSize = self.resizeHandleSize

        # bunlarin yeri onemli degil, QSize(0,0) da olur, asagida yerlerine kaydiriliorlar
        self.topLeftHandle = QRectF(self._rect.topLeft(), resizeHandleSize)
        self.topRightHandle = QRectF(self._rect.topRight(), resizeHandleSize)
        self.bottomRightHandle = QRectF(self._rect.bottomRight(), resizeHandleSize)
        self.bottomLeftHandle = QRectF(self._rect.bottomLeft(), resizeHandleSize)

        # # self.topLeftHandle.moveTopLeft(self._rect.topLeft())
        # self.topRightHandle.moveTopRight(self._rect.topRight())
        # self.bottomRightHandle.moveBottomRight(self._rect.bottomRight())
        # self.bottomLeftHandle.moveBottomLeft(self._rect.bottomLeft())

        self.topLeftHandle.moveCenter(self._rect.topLeft())
        self.topRightHandle.moveCenter(self._rect.topRight())
        self.bottomRightHandle.moveCenter(self._rect.bottomRight())
        self.bottomLeftHandle.moveCenter(self._rect.bottomLeft())

    # ---------------------------------------------------------------------
    def sceneCenter(self):
        if self.parentItem():
            return self.mapToParent(self.boundingRect().center())
        return self.mapToScene(self.boundingRect().center())

    # ---------------------------------------------------------------------
    def sceneWidth(self):
        return self.sceneRight() - self.sceneLeft()

    # ---------------------------------------------------------------------
    def sceneHeight(self):
        return self.sceneBottom() - self.sceneTop()

    # ---------------------------------------------------------------------
    def sceneRight(self):
        return max(self.sceneBoundingRect().topRight().x(),
                   self.sceneBoundingRect().bottomRight().x())

    # ---------------------------------------------------------------------
    def sceneLeft(self):
        return min(self.sceneBoundingRect().topLeft().x(),
                   self.sceneBoundingRect().bottomLeft().x())

    # ---------------------------------------------------------------------
    def sceneTop(self):
        return min(self.sceneBoundingRect().topLeft().y(),
                   self.sceneBoundingRect().topRight().y())

    # ---------------------------------------------------------------------
    def sceneBottom(self):
        return max(self.sceneBoundingRect().bottomRight().y(),
                   self.sceneBoundingRect().bottomLeft().y())

    # ---------------------------------------------------------------------
    def setSceneLeft(self, left):
        self.setX(left + self.scenePos().x() - self.sceneLeft())

    # ---------------------------------------------------------------------
    def setSceneRight(self, right):
        self.setX(right + self.scenePos().x() - self.sceneRight())

    # ---------------------------------------------------------------------
    def setSceneTop(self, top):
        self.setY(top + self.scenePos().y() - self.sceneTop())

    # ---------------------------------------------------------------------
    def setSceneBottom(self, bottom):
        self.setY(bottom + self.scenePos().y() - self.sceneBottom())

    # ---------------------------------------------------------------------
    def setHorizontalCenter(self, hcenter):
        self.setY(hcenter + self.scenePos().y() - self.sceneCenter().y())

    # ---------------------------------------------------------------------
    def setVerticalCenter(self, vcenter):
        self.setX(vcenter + self.scenePos().x() - self.sceneCenter().x())

    # ---------------------------------------------------------------------
    def flipHorizontal(self, mposx):
        ipos = self.scenePos()
        rc = self.mapToScene(self.boundingRect().center())
        diff = mposx - rc.x()

        ipos.setX(ipos.x() + 2 * diff)
        if self.parentItem():
            # ipos = (ipos - self.parentItem().scenePos())
            ipos = (self.parentItem().mapFromScene(ipos))
        self.setPos(ipos)
        if self.rotation():
            self.rotateWithOffset(360 - self.rotation())

        eskiRot = self.rotation()
        if eskiRot:
            self.rotateWithOffset(0)
        for c in self.childItems():
            c.flipHorizontal(self.sceneCenter().x())
        if eskiRot:
            self.rotateWithOffset(eskiRot)

    # ---------------------------------------------------------------------
    def flipVertical(self, mposy):
        ipos = self.scenePos()
        rc = self.mapToScene(self.boundingRect().center())
        diff = mposy - rc.y()

        ipos.setY(ipos.y() + 2 * diff)
        if self.parentItem():
            # ipos = (ipos - self.parentItem().scenePos())
            ipos = (self.parentItem().mapFromScene(ipos))
        self.setPos(ipos)
        if self.rotation():
            self.rotateWithOffset(360 - self.rotation())

        eskiRot = self.rotation()
        if eskiRot:
            self.rotateWithOffset(0)
        for c in self.childItems():
            c.flipVertical(self.sceneCenter().y())
        if eskiRot:
            self.rotateWithOffset(eskiRot)

    # ---------------------------------------------------------------------
    def contextMenuEvent(self, event):
        # TODO: bu alttaki iki satir aktif idi, ve de ctrl harici sag tiklayinca
        # (base- text -path -group nesnelerinin contextMenuEventinde var)
        # mesela birden fazla nesne secili ve de gruplayacagız diyelim sag menu ile
        # ctrl basılı degil ise tikladigimiz secili kaliyor digerleri siliniyordu
        # uygunsuz bir kullanıcı deneyimi, niye yaptık acaba boyleyi hatırlayana kadar kalsın burda :)
        # if not event.modifiers() == Qt.KeyboardModifier.ControlModifier:
        #     self.scene().clearSelection()
        self.setSelected(True)

        # self.scene().parent().item_context_menu(self, event.screenPos())
        self.scene().parent().on_yazi_sag_menu_about_to_show(self)
        self.scene().parent().yaziSagMenu.popup(event.screenPos())

        event.accept()

    # ---------------------------------------------------------------------
    def setText(self, text):
        self.setPlainText(text)

    # ---------------------------------------------------------------------
    def text(self):
        return self.toPlainText()

    # ---------------------------------------------------------------------
    def setCommand(self, title, command):
        self._command["title"] = title
        self._command["command"] = command

    # ---------------------------------------------------------------------
    def command(self):
        return self._command

    # ---------------------------------------------------------------------
    def setFont(self, font, yaziTipiMiDegisti=True):
        if yaziTipiMiDegisti:
            font.setBold(self.font().bold())
            font.setItalic(self.font().italic())
            font.setUnderline(self.font().underline())
            font.setStrikeOut(self.font().strikeOut())
            font.setOverline(self.font().overline())
            # font.setPointSizeF(self.font().pointSizeF())
        super(Text, self).setFont(font)
        if self.parentItem():
            self.yazi_kutusunu_daralt()

    # ---------------------------------------------------------------------
    def setFontPointSizeF(self, fontPointSizeF):
        font = self.font()
        font.setPointSizeF(fontPointSizeF)
        self.setFont(font, yaziTipiMiDegisti=False)
        self.update()

    # ---------------------------------------------------------------------
    def fontPointSizeF(self):
        return self.font().pointSizeF()

    # ---------------------------------------------------------------------
    def ver_yazi_hizasi(self):
        return self.doc.defaultTextOption().alignment()

    # ---------------------------------------------------------------------
    def kur_yazi_hizasi(self, hiza):
        option = self.doc.defaultTextOption()
        option.setAlignment(hiza)
        self.doc.setDefaultTextOption(option)

    # ---------------------------------------------------------------------
    def ver_karakter_bicimi(self):
        # return self._font.bold | self._font.italic | self._font.under | self._font.strikeout
        font = self.font()
        return {"b": font.bold(),
                "i": font.italic(),
                "u": font.underline(),
                "s": font.strikeOut(),
                "o": font.overline(),
                }

    # ---------------------------------------------------------------------
    def kur_karakter_bicimi(self, sozluk):
        font = self.font()
        font.setBold(sozluk["b"])
        font.setItalic(sozluk["i"])
        font.setUnderline(sozluk["u"])
        font.setStrikeOut(sozluk["s"])
        font.setOverline(sozluk["o"])
        self.setFont(font, yaziTipiMiDegisti=False)
        # self.update()

    # ---------------------------------------------------------------------
    def setPen(self, pen):

        if self._pen == pen:
            return
        self.prepareGeometryChange()
        self._pen = pen
        # self._boundingRect = QRectF()
        self.update(self.boundingRect())

    # ---------------------------------------------------------------------
    def pen(self):
        return self._pen

    # ---------------------------------------------------------------------
    def setBrush(self, brush):

        if self._brush == brush:
            return
        self._brush = brush
        self.update()

    # ---------------------------------------------------------------------
    def brush(self):
        return self._brush

    # ---------------------------------------------------------------------
    def setCizgiRengi(self, col):

        _selectionLineBgColor = QColor.fromHsv(col.hue(),
                                               0,
                                               20 if col.value() > 127 else 250)

        _activeItemLineColor = QColor(self.activeItemLineColor)
        if col.hue() > 300 or col.hue() < 30:
            _activeItemLineColor.setHsv((col.hue() + 150) % 360,
                                        self.activeItemLineColor.saturation(),
                                        self.activeItemLineColor.value())
        self._pen.setColor(col)
        # self.setDefaultTextColor(col)
        self.cizgiRengi = col

        self.selectionPenBottom = QPen(_selectionLineBgColor,
                                       self.secili_nesne_kalem_kalinligi,
                                       Qt.PenStyle.DashLine,
                                       Qt.PenCapStyle.RoundCap, Qt.PenJoinStyle.RoundJoin)
        self.selectionPenBottomIfAlsoActiveItem = QPen(_activeItemLineColor,
                                                       self.secili_nesne_kalem_kalinligi,
                                                       Qt.PenStyle.DashLine,
                                                       Qt.PenCapStyle.RoundCap, Qt.PenJoinStyle.RoundJoin)

        self.update()

    # ---------------------------------------------------------------------
    def setYaziRengi(self, col):

        self.yaziRengi = col
        # self.textPen = QPen(QColor().fromRgb(col.red(), col.green(), col.blue(), col.alpha()))
        self.setDefaultTextColor(col)
        self.update()

    # ---------------------------------------------------------------------
    def setCizgiKalinligi(self, width):
        self._pen.setWidthF(width)
        self.update()

    # ---------------------------------------------------------------------
    def setArkaPlanRengi(self, col):
        self.arkaPlanRengi = col
        self.setBrush(QBrush(col))
        if col.value() > 245:
            self.handleBrush = shared.handleBrushKoyu
        else:
            self.handleBrush = shared.handleBrushAcik

    # ---------------------------------------------------------------------
    def itemChange(self, change, value):

        if change == QGraphicsTextItem.GraphicsItemChange.ItemSelectedChange:
            if value:
                self.scene().parent().text_item_selected(self)
            else:
                self.scene().parent().text_item_deselected(self)

        # return QGraphicsRectItem.itemChange(self, change, value)
        return value
        # super(Rect, self).itemChange(change, value)

    # ---------------------------------------------------------------------
    def focusInEvent(self, event):

        if self.scene():
            if self.doc.availableUndoSteps():
                self.scene().parent().log(
                    self.tr("{} undo(s), {} redo(s) available for items' text.").format(self.doc.availableUndoSteps(),
                                                                                        self.doc.availableRedoSteps()),
                    4500,
                    toStatusBarOnly=True)

        super(Text, self).focusInEvent(event)

    # ---------------------------------------------------------------------
    def focusOutEvent(self, event):

        self.textItemFocusedOut.emit(self)
        if self.scene():
            self.scene().parent()._statusBar.clearMessage()

        # self.setTextInteractionFlags(Qt.TextSelectableByMouse)
        # self.setTextInteractionFlags(Qt.TextInteractionFlag.NoTextInteraction)
        # self.clear_selection()
        if self.textCursor().hasSelection():
            return QGraphicsTextItem.focusOutEvent(self, event)

        super(Text, self).focusOutEvent(event)

    # ---------------------------------------------------------------------
    def clear_selection(self):
        cursor = self.textCursor()
        # self.cursor_eski_pozisyon = cursor.position()
        # self.cursor_eski_anchor = cursor.anchor()
        cursor.clearSelection()
        self.setTextCursor(cursor)

    # ---------------------------------------------------------------------
    def mouseDoubleClickEvent(self, event):
        # if event.modifiers() == Qt.KeyboardModifier.ControlModifier:
        #     self.scene().parent().web_browser_ac(self.text())
        # elif event.modifiers() == Qt.KeyboardModifier.ControlModifier | Qt.KeyboardModifier.ShiftModifier:
        #     self.scene().parent().web_browser_ac("https://www.google.com/search?q=" + self.text())
        if event.modifiers() == Qt.KeyboardModifier.ControlModifier:
            self.yazi_kutusunu_daralt()

        # elif event.modifiers() == Qt.KeyboardModifier.ShiftModifier:
        #     self.setTextInteractionFlags(Qt.TextInteractionFlag.NoTextInteraction)
        #     self.clear_selection()

        else:
            # if self.textInteractionFlags() == Qt.NoTextInteraction:
            self.setTextInteractionFlags(Qt.TextInteractionFlag.TextEditorInteraction)
            self.setFocus()
            # self.clearFocus()
        super(Text, self).mouseDoubleClickEvent(event)

    # ---------------------------------------------------------------------
    def hoverEnterEvent(self, event):
        # event propagationdan dolayi bos da olsa burda implement etmek lazim
        # mousePress,release,move,doubleclick de bu mantıkta...
        # baska yerden baslayip mousepress ile , mousemove baska, mouseReleas baska widgetta gibi
        # icinden cikilmaz durumlara sebep olabiliyor.
        # ayrica override edince accept() de edilmis oluyor mouseeventler
        super(Text, self).hoverEnterEvent(event)

    # ---------------------------------------------------------------------
    def hoverMoveEvent(self, event):

        # cursor = self.scene().parent().cursor()

        # if self.isSelected() and self.scene().aktifArac == self.scene().SecimAraci:
        if not self.ustGrup:
            if self.scene().aktifArac == self.scene().SecimAraci:
                if self.topLeftHandle.contains(event.pos()) or self.bottomRightHandle.contains(event.pos()):
                    self.scene().parent().setCursor(Qt.CursorShape.SizeFDiagCursor, gecici_mi=True)
                    # self.setCursor(Qt.SizeFDiagCursor, gecici_mi=True)
                elif self.topRightHandle.contains(event.pos()) or self.bottomLeftHandle.contains(event.pos()):
                    self.scene().parent().setCursor(Qt.CursorShape.SizeBDiagCursor, gecici_mi=True)
                    # self.setCursor(Qt.SizeBDiagCursor, gecici_mi=True)
                else:
                    self.scene().parent().setCursor(self.scene().parent().imlec_arac, gecici_mi=True)

        super(Text, self).hoverMoveEvent(event)

    # ---------------------------------------------------------------------
    def hoverLeaveEvent(self, event):
        # if self.isSelected():
        # self.setCursor(self.saved_cursor)
        if not self.ustGrup:
            self.scene().parent().setCursor(self.scene().parent().imlec_arac, gecici_mi=True)

        super(Text, self).hoverLeaveEvent(event)

    # ---------------------------------------------------------------------
    def wheelEvent(self, event):

        if self.ustGrup:
            return QGraphicsTextItem.wheelEvent(self, event)

        toplam = event.modifiers().value

        # ctrl = int(Qt.ControlModifier)
        shift = Qt.KeyboardModifier.ShiftModifier.value
        alt = Qt.KeyboardModifier.AltModifier.value

        ctrlAlt = Qt.KeyboardModifier.ControlModifier.value + Qt.KeyboardModifier.AltModifier.value
        ctrlShift = Qt.KeyboardModifier.ControlModifier.value + Qt.KeyboardModifier.ShiftModifier.value
        altShift = Qt.KeyboardModifier.AltModifier.value + Qt.KeyboardModifier.ShiftModifier.value
        ctrlAltShift = Qt.KeyboardModifier.ControlModifier.value + Qt.KeyboardModifier.AltModifier.value + Qt.KeyboardModifier.ShiftModifier.value

        # if event.modifiers() & Qt.ControlModifier:
        if toplam == ctrlShift:
            # self.scaleItem(event.delta())
            self.scaleItemWithFontSizeF(event.delta())
            # self.scaleItem(event.angleDelta().y())

        # elif event.modifiers() & Qt.ShiftModifier:
        elif toplam == shift:
            self.changeFontSizeF(event.delta())

        # elif event.modifiers() & Qt.AltModifier:
        elif toplam == alt:
            self.changeBackgroundColorAlpha(event.delta())

        elif toplam == ctrlAlt:
            self.changeTextColorAlpha(event.delta())

        elif toplam == ctrlAltShift:
            self.rotateItem(event.delta())

        elif toplam == altShift:
            # self.changeImageItemTextBackgroundColorAlpha(event.delta())
            self.changeLineColorAlpha(event.delta())

        # elif toplam == ctrlAltShift:
        #     self.scaleItemWithFontSizeF(event.delta())

        else:
            super(Text, self).wheelEvent(event)

    # ---------------------------------------------------------------------
    def scaleItemWithFontSizeF2(self, delta):
        self.changeFontSizeF(delta)

    # ---------------------------------------------------------------------
    def scaleItemWithFontSizeF(self, delta):

        scaleFactor = 1.1
        if delta < 0:
            scaleFactor = 1 / scaleFactor
            # scaleFactor = 0.91

        # self.setTransformOriginPoint(self.boundingRect().center())
        # fontPointSizeF = max(3, (self.fontPointSizeF() * scaleFactor))
        fontPointSizeF = self.fontPointSizeF() * scaleFactor

        # eskiTamCerceve = self.boundingRect() | self.childrenBoundingRect()
        # yeniTamCerceveSize = eskiTamCerceve.size() * scaleFactor

        self.scene().undoRedo.undoableScaleTextItemByResizing(self.scene().undoStack,
                                                              "_scale by resizing",
                                                              self, scaleFactor, fontPointSizeF
                                                              )

        self.yazi_kutusunu_daralt()

    # ---------------------------------------------------------------------
    def changeFontSizeF(self, delta):
        # font = self.font()
        size = self.fontPointSizeF()

        if delta > 0:
            # font.setPointSizeF(size + 1)
            size += 1

        else:
            if size > 3:
                # font.setPointSizeF(size - 1)
                size -= 1
            else:
                # undolari biriktermesin diye donuyoruz,
                # yoksa zaten ayni size de yeni bir undolu size komutu veriyor.
                return

        self.scene().undoRedo.undoableSetFontSizeF(self.scene().undoStack, "change text size", self, size)
        self.yazi_kutusunu_daralt()
        # TODO: child items baseden bak

    # ---------------------------------------------------------------------
    def rotateWithOffset(self, angle):
        cEski = self.sceneCenter()
        self.setRotation(angle)
        cYeni = self.sceneCenter()
        diff = cEski - cYeni
        self.moveBy(diff.x(), diff.y())

    # ---------------------------------------------------------------------
    def rotateItem(self, delta):

        # self.setTransformOriginPoint(self.boundingRect().center())
        if delta > 0:
            # self.setRotation(self.rotation() + 5)
            self.scene().undoRedo.undoableRotateWithOffset(self.scene().undoStack, "rotate", self, self.rotation() + 5)
        else:
            # self.setRotation(self.rotation() - 5)
            self.scene().undoRedo.undoableRotateWithOffset(self.scene().undoStack, "rotate", self, self.rotation() - 5)
        # self.update()

    # ---------------------------------------------------------------------
    def changeBackgroundColorAlpha(self, delta):

        col = shared.calculate_alpha(delta, QColor(self.arkaPlanRengi))

        # self.setBrush(self.arkaPlanRengi)
        # self.update()
        # self.scene().undoableSetItemBackgroundColorAlpha("Change item's background alpha", self, col)
        if self.childItems():
            self.scene().undoStack.beginMacro("change items' background alpha")
            self.scene().undoRedo.undoableSetItemBackgroundColorAlpha(self.scene().undoStack,
                                                                      "_change item's background alpha",
                                                                      self, col)
            for c in self.childItems():
                c.changeBackgroundColorAlpha(delta)
            self.scene().undoStack.endMacro()
        else:
            self.scene().undoRedo.undoableSetItemBackgroundColorAlpha(self.scene().undoStack,
                                                                      "change item's background alpha", self,
                                                                      col)

    # ---------------------------------------------------------------------
    def changeLineColorAlpha(self, delta):

        col = shared.calculate_alpha(delta, self.defaultTextColor())

        # self.setDefaultTextColor(col)
        # self.scene().undoableSetLineOrTextColorAlpha("Change item's text alpha", self, col)
        if self.childItems():
            self.scene().undoStack.beginMacro("change items' line alpha")
            self.scene().undoRedo.undoableSetLineColorAlpha(self.scene().undoStack, "_change item's line alpha", self,
                                                            col)
            for c in self.childItems():
                c.changeLineColorAlpha(delta)
            self.scene().undoStack.endMacro()
        else:
            self.scene().undoRedo.undoableSetLineColorAlpha(self.scene().undoStack, "change item's line alpha", self,
                                                            col)

    # ---------------------------------------------------------------------
    def changeTextColorAlpha(self, delta):

        col = shared.calculate_alpha(delta, self.yaziRengi)

        if self.childItems():
            self.scene().undoStack.beginMacro("change items' text alpha")
            self.scene().undoRedo.undoableSetTextColorAlpha(self.scene().undoStack, "_change item's text alpha", self,
                                                            col)
            for c in self.childItems():
                c.changeTextColorAlpha(delta)
            self.scene().undoStack.endMacro()
        else:
            self.scene().undoRedo.undoableSetTextColorAlpha(self.scene().undoStack, "change item's text alpha", self,
                                                            col)

    # ---------------------------------------------------------------------
    def changeImageItemTextBackgroundColorAlpha(self, delta):
        # grup nesnesinden cagriliyor alt+shift
        if self.childItems():
            startedMacro = False
            for c in self.childItems():
                if c.type() == shared.IMAGE_ITEM_TYPE:
                    if not startedMacro:
                        self.scene().undoStack.beginMacro("change image items' text background alpha")
                        startedMacro = True
                    c.changeImageItemTextBackgroundColorAlpha(delta)
            if startedMacro:
                self.scene().undoStack.endMacro()

    # ---------------------------------------------------------------------
    def mousePressEvent(self, event):

        self._resizing = False  # we could use "self._resizingFrom = 0" instead, but self._resizing is more explicit.
        self._fixedResizePoint = None
        self._resizingFrom = None
        self._eskiRectBeforeResize = None

        self._eskiPosBeforeResize = self.pos()
        # self._eskiPosBeforeResize = self.scenePos()

        if self.topLeftHandle.contains(event.pos()):
            self._resizing = True
            self._fixedResizePoint = self._rect.bottomRight()
            self._resizingFrom = 1
            self._eskiRectBeforeResize = QRectF(self._rect)
        elif self.topRightHandle.contains(event.pos()):
            self._resizing = True
            self._fixedResizePoint = self._rect.bottomLeft()
            self._resizingFrom = 2
            self._eskiRectBeforeResize = QRectF(self._rect)
        elif self.bottomRightHandle.contains(event.pos()):
            self._resizing = True
            self._fixedResizePoint = self._rect.topLeft()
            self._resizingFrom = 3
            self._eskiRectBeforeResize = QRectF(self._rect)
        elif self.bottomLeftHandle.contains(event.pos()):
            self._resizing = True
            self._fixedResizePoint = self._rect.topRight()
            self._resizingFrom = 4
            self._eskiRectBeforeResize = QRectF(self._rect)

        super(Text, self).mousePressEvent(event)

        # TODO: bu kozmetik amacli, sadece release de olunca gecikmeli seciyormus hissi oluyor
        if self.isSelected() and self.childItems():
            self.scene().select_all_children_recursively(self, cosmeticSelect=True)

        # self.scene().deactivate_item(self)
        # self.scene().activate_item(self)

    # ---------------------------------------------------------------------
    def mouseReleaseEvent(self, event):
        if self._resizing:
            # sahneye ilk cizim kontrolu,(undoya eklemesin diye)
            # if not self._eskiRectBeforeResize.size() == QSizeF(1, 1):
            #     rect = self.boundingRect()
            #     self.setPos(self.mapToScene(rect.topLeft()))
            #     rect.moveTo(0, 0)
            #     self.scene().undoableResizeBaseItem("resize",
            #                                         self,
            #                                         # yeniRect=self._rect,
            #                                         yeniRect=rect,
            #                                         eskiRect=self._eskiRectBeforeResize,
            #                                         eskiPos=self._eskiPosBeforeResize)

            # TODO: bu satir eger cizim disari tasarsa, scene recti ona uygun ayarlamak icin
            # fakat eger cizim disari tasarsa evet uyarliyor ama bazen de çizilen itemin geometrisini degistiriyor.
            # o yuzden simdilik iptal.
            # self.scene().unite_with_scene_rect(self.sceneBoundingRect())

            # rect = self.boundingRect()
            rect = self._rect
            # self.setPos(self.mapToScene(rect.topLeft()))
            # rect.moveTo(0, 0)
            # self.doc.setTextWidth(rect.width())

            # undo icinde
            # self.update_resize_handles()

            self.scene().undoRedo.undoableResizeBaseItem(self.scene().undoStack,
                                                         "resize",
                                                         self,
                                                         # yeniRect=self._rect,
                                                         yeniRect=rect,
                                                         eskiRect=self._eskiRectBeforeResize,
                                                         eskiPos=self._eskiPosBeforeResize)

            # print(self.mapRectToScene(self._eskiRectBeforeResize))
            # self.scene().update(self.mapRectToScene(self._eskiRectBeforeResize))
            # self.scene().views()[0].updateScene([self.mapRectToScene(self._eskiRectBeforeResize)])
            self._resizing = False
            self.scene().unite_with_scene_rect(self.sceneBoundingRect())

        super(Text, self).mouseReleaseEvent(event)

        if self.childItems():
            if self.isSelected():
                self.scene().select_all_children_recursively(self, cosmeticSelect=False, topmostParent=True)
            else:
                self.scene().select_all_children_recursively(self, cosmeticSelect=False)

    # ---------------------------------------------------------------------
    def mouseMoveEvent(self, event):

        if self._resizing:

            px = self._fixedResizePoint.x()
            py = self._fixedResizePoint.y()

            # mx = event.scenePos().x()
            # my = event.scenePos().y()
            mx = event.pos().x()
            my = event.pos().y()
            topLeft = QPointF(min(px, mx), min(py, my))  # top-left corner (x,y)
            bottomRight = QPointF(max(px, mx), max(py, my))  # bottom-right corner (x,y)
            # size = QSizeF(fabs(mx - px), fabs(my - py))
            # rect = QRectF(topLeft, size)
            # rect = QRectF(QPointF(0, 0), size)

            rect = QRectF(topLeft, bottomRight)
            # rect = rect.normalized()

            # self.setPos(self.mapToScene(topLeft))
            # self.setPos(self.mapFromScene(topLeft))
            # eger bu move to 0,0 koymazsak o zaman normal resize ediyoruz ama
            # sol taraftan resize edersek release edince text tepki veriyor.
            rect.moveTo(0, 0)
            self.setRect(rect)
            # self.setPos(self.mapToScene(rect.topLeft()))
            self.scene().parent().change_transform_box_values(self)

        # event.accept()
        else:
            super(Text, self).mouseMoveEvent(event)

        # super(BaseItem, self).mouseMoveEvent(event)

    # ---------------------------------------------------------------------
    def yazi_kutusunu_daralt(self):
        # self._rect.setSize(QSizeF(self.doc.size()))
        r = QRectF(self._rect)
        # r.setSize(QSizeF(self.textWidth(), self.doc.size().height()))

        fm = QFontMetricsF(self.font())
        # yaziGenislik = fm.horizontalAdvance(self.text(), QTextOption())
        yaziGenislik = fm.boundingRect(r, 0, self.text()).size().width()
        # yaziYukseklik = fm.height()
        # self.doc.adjustSize()
        # self.doc.setTextWidth(yaziGenislik + 10)
        r.setWidth(yaziGenislik + 10)
        r.setHeight(self.doc.size().height())
        # r.setSize(QSizeF(self.doc.size()))
        self.setRect(r)

    # ---------------------------------------------------------------------
    def setRect(self, rect):
        if self._rect == rect:
            return

        self.prepareGeometryChange()
        self._rect = rect
        # TODO: burda direkt self.settextwidth niye demedik ki
        self.doc.setTextWidth(rect.width())
        # self.doc.setPageSize(rect.size())
        self._boundingRect = rect.adjusted(-self.boundingRectTasmaDegeri, -self.boundingRectTasmaDegeri,
                                           self.boundingRectTasmaDegeri, self.boundingRectTasmaDegeri)
        self.update_resize_handles()
        # self.adjustSize()
        # self.scene().parent().change_transform_box_values(self)
        # self._boundingRect = QRectF()

        if self.doc.size().height() > rect.size().height():
            self.isTextOverflowed = True
        else:
            self.isTextOverflowed = False
        self.update()

    # ---------------------------------------------------------------------
    def rect(self):
        return self._rect

    # ---------------------------------------------------------------------
    def shape(self):

        # if (!dd->control)
        # return QPainterPath();
        #
        # QPainterPath path;
        # path.addRect(dd->boundingRect);
        # return path;

        # path = QPainterPath()
        # # path.addRect(self._rect)
        # if self._rect.isEmpty():
        #     return super(Text, self).shape()
        # else:
        #     path.addRect(self._rect)
        # # return self.qt_graphicsItem_shapeFromPath(path, self._pen)
        #     return path

        path = QPainterPath()
        # path.addRect(self._rect)
        path.addRect(self._boundingRect)
        return path

    # ---------------------------------------------------------------------
    def boundingRect(self):
        # return self._rect
        return self._boundingRect
        # br = QRectF(self.rect())

    # void QGraphicsTextItemPrivate::_q_updateBoundingRect(const QSizeF &size)
    # {
    #     if (!control) return; // can't happen
    #     const QSizeF pageSize = control->document()->pageSize();
    #     // paged items have a constant (page) size
    #     if (size == boundingRect.size() || pageSize.height() != -1)
    #         return;
    #     qq->prepareGeometryChange();
    #     boundingRect.setSize(size);
    #     qq->update();
    # }

    # # ---------------------------------------------------------------------
    # def qt_graphicsItem_shapeFromPath(self, path, pen):
    #     # We unfortunately need this hack as QPainterPathStroker will set a width of 1.0
    #     # if we pass a value of 0.0 to QPainterPathStroker.setWidth()
    #     penWidthZero = 0.00000001
    #     if path == QPainterPath():
    #         return path
    #     ps = QPainterPathStroker()
    #     ps.setCapStyle(pen.capStyle())
    #
    #     if pen.widthF() <= 0:
    #         ps.setWidth(penWidthZero)
    #     else:
    #         ps.setWidth(pen.widthF())
    #     ps.setJoinStyle(pen.joinStyle())
    #     ps.setMiterLimit(pen.miterLimit())
    #     p = ps.createStroke(path)  # return type: QPainterPath
    #     p.addPath(path)
    #     return p

    # ---------------------------------------------------------------------
    #         inline QPointF controlOffset() const
    #         {
    #         return QPointF(0., pageNumber * control->document()->pageSize().height());}

    # ---------------------------------------------------------------------
    #     void
    #     QGraphicsTextItem::paint(QPainter * painter, const
    #     QStyleOptionGraphicsItem * option,
    #     QWidget * widget)
    #
    # {
    #     Q_UNUSED(widget);
    # if (dd->control) {
    # painter->save();
    # QRectF r = option->exposedRect;
    # painter->translate(-dd->controlOffset());
    # r.translate(dd->controlOffset());
    #
    # QTextDocument * doc = dd->control->document();
    # QTextDocumentLayout * layout = qobject_cast < QTextDocumentLayout * > (doc->documentLayout());
    #
    # // the layout might need to expand the root frame to
    # // the viewport if NoWrap is set
    # if (layout)
    # layout->setViewport(dd->boundingRect);
    #
    # dd->control->drawContents(painter, r);
    #
    # if (layout)
    # layout->setViewport(QRect());
    #
    # painter->restore();
    # }
    #
    # if (option->state & (QStyle::
    #     State_Selected | QStyle::State_HasFocus))
    # qt_graphicsItem_highlightSelected(this, painter, option);
    # }
    # ---------------------------------------------------------------------
    def keyPressEvent(self, event):

        if event.key() == Qt.Key.Key_Escape:
            self.clearFocus()
            # bu asagidaki if var cunku text item bos ise
            # birden fazla defa selectionqueue da remove etmeye calisabiliyor
            # veya item sahneden silindiginden item.scene() none donuyor gibi sebeplerden dolayi.
            # self.childItems() kontrolu de, yazi bir baska nesnenin parenti iken silinmis olabilir diye.
            if self.toPlainText() or self.childItems():
                self.setSelected(False)

        if event.modifiers() == Qt.KeyboardModifier.ControlModifier:
            if event.key() == Qt.Key.Key_Enter or event.key() == Qt.Key.Key_Return:
                self.clearFocus()
                if self.toPlainText() or self.childItems():
                    self.setSelected(False)

        if event.modifiers() == Qt.KeyboardModifier.ShiftModifier:
            if event.key() == Qt.Key.Key_Enter or event.key() == Qt.Key.Key_Return:
                self.clearFocus()
                if self.toPlainText() or self.childItems():
                    self.setSelected(False)
                    self.scene().create_empty_text_object_with_double_click(
                        QPointF(self.sceneLeft() + self.boundingRectTasmaDegeri, self.sceneBottom() + 10))

        super(Text, self).keyPressEvent(event)

    # ---------------------------------------------------------------------
    def html_dive_cevir(self, html_klasor_kayit_adres, dosya_kopyalaniyor_mu):
        rect = self.sceneBoundingRect()

        x = self.scenePos().x()
        y = self.scenePos().y()
        xs = self.scene().sceneRect().x()
        ys = self.scene().sceneRect().y()
        x = x - xs
        y = y - ys

        w = rect.width()
        h = rect.height()

        bicimSozluk = self.ver_karakter_bicimi()
        bold = "font-weight:bold;" if bicimSozluk["b"] else ""
        italic = "font-style:italic;" if bicimSozluk["i"] else ""
        underline = "underline" if bicimSozluk["u"] else ""
        strikeOut = "line-through" if bicimSozluk["s"] else ""
        overline = "overline" if bicimSozluk["o"] else ""
        bicimler1 = bold + italic
        if any((underline, strikeOut, overline)):
            bicimler2 = f"text-decoration: {underline} {strikeOut} {overline};"
        else:
            bicimler2 = ""

        renk_arkaPlan = f"({self.arkaPlanRengi.red()},{self.arkaPlanRengi.green()},{self.arkaPlanRengi.blue()},{self.arkaPlanRengi.alpha() / 255})"
        renk_yazi = f"({self.yaziRengi.red()},{self.yaziRengi.green()},{self.yaziRengi.blue()},{self.yaziRengi.alpha() / 255})"

        if self.isPlainText:
            yazi_str = f'<p style=" display: inline-block; ' \
                       f'color:rgba{renk_yazi}; ' \
                       f'margin:0; padding:7px;">{self.text()}</p>'
        else:
            html = self.toHtml()
            body_baslangic = html.find("<bo")
            bas = html.find('">', body_baslangic)
            son = html.find("</bo")
            # yazi = self.toHtml()
            yazi_str = html[bas + 2:son]

        hiza = self.ver_yazi_hizasi()
        # if hiza == Qt.AlignLeft or hiza == Qt.AlignLeft | Qt.AlignVCenter:
        #     yazi_hiza = "left"
        if hiza == Qt.AlignmentFlag.AlignCenter or hiza == Qt.AlignmentFlag.AlignCenter | Qt.AlignmentFlag.AlignVCenter:
            yazi_hiza = "center"
        elif hiza == Qt.AlignmentFlag.AlignRight or hiza == Qt.AlignmentFlag.AlignRight | Qt.AlignmentFlag.AlignVCenter:
            yazi_hiza = "right"
        elif hiza == Qt.AlignmentFlag.AlignJustify or hiza == Qt.AlignmentFlag.AlignJustify | Qt.AlignmentFlag.AlignVCenter:
            yazi_hiza = "justify"
        else:
            yazi_hiza = "left"

        div_str = f"""
                    <article style=" 
                     color:rgba{renk_yazi};
                     background:rgba{renk_arkaPlan};
                     font-size:{self.fontPointSizeF()}pt; 
                     font-family:{self.font().family()};
                     text-align: {yazi_hiza};
                     {bicimler1}
                     {bicimler2}
                     position:absolute;
                     z-index:{int(self.zValue() * 10) if self.zValue() else 0};
                     top:{y}px;
                     left:{x}px;
                     width:{w};
                     height:{h};
                     transform-box: fill-box;
                     transform-origin: top left;
                     transform: rotate({self.rotation()}deg);

                     " id="{self._kim}">{yazi_str}</article>\n
            """

        return div_str

    # ---------------------------------------------------------------------
    def paint(self, painter, option, widget):

        painter.save()
        # painter.fillRect(option.rect, self.arkaPlanRengi)
        painter.fillRect(option.rect, self.brush())

        if option.state & QStyle.StateFlag.State_Selected or self.cosmeticSelect:
            if self.isActiveItem:
                selectionPenBottom = self.selectionPenBottomIfAlsoActiveItem
                if not self.isPinned:
                    painter.setPen(Qt.PenStyle.NoPen)
                    painter.setBrush(self.handleBrush)
                    painter.drawRect(self.topLeftHandle)
                    painter.drawRect(self.topRightHandle)
                    painter.drawRect(self.bottomRightHandle)
                    painter.drawRect(self.bottomLeftHandle)
            else:
                selectionPenBottom = self.selectionPenBottom
            painter.setPen(selectionPenBottom)
            painter.setBrush(Qt.BrushStyle.NoBrush)
            painter.drawRect(option.rect)

            ########################################################################
            # !!! simdilik iptal, gorsel fazlalik olusturmakta !!!
            ########################################################################
            # if not self.isPinned and self.isActiveItem:
            # painter.setPen(Qt.PenStyle.NoPen)
            # painter.setBrush(self.handleBrush)
            # painter.drawRect(self.topLeftHandle)
            # painter.drawRect(self.topRightHandle)
            # painter.drawRect(self.bottomRightHandle)
            # painter.drawRect(self.bottomLeftHandle)
            ########################################################################

        # if option.state & QStyle.StateFlag.State_MouseOver:
        #     painter.setBrush(Qt.BrushStyle.NoBrush)
        #     painter.setPen(self.selectionPenBottom)
        #     # painter.setPen(QPen(Qt.GlobalColor.red))
        #     painter.drawRect(option.rect)

        if self.isTextOverflowed:
            painter.setPen(QPen(Qt.GlobalColor.red, 2, Qt.PenStyle.DotLine))
            painter.drawLine(self._boundingRect.bottomLeft(), self._boundingRect.bottomRight())

        # if not self.isPlainText:
        #     # painter.drawEllipse(self.zenginYaziBelirteci)
        #     # painter.drawText(5, 5, "📌🎨☼☀®")
        #     painter.drawText(0, 10, "*")

        # painter.setPen(Qt.blue)
        # painter.drawRect(self.boundingRect())
        # painter.setPen(Qt.green)
        # painter.drawRect(self._rect)
        painter.restore()

        # option2 = QStyleOptionGraphicsItem(option)
        # option2.state = 0
        # option.state &= ~(QStyle.StateFlag.State_Selected | QStyle.StateFlag.State_HasFocus)
        # option2.exposedRect.setSize(self.document().pageSize())
        # # option2.exposedRect.setSize(self.document().documentLayout().documentSize())
        #
        # option.exposedRect = self.boundingRect()
        # option.rect = option.exposedRect.toRect()
        #
        # rect = QRectF(self._rect)
        # rect.moveTo(0, 0)
        # option2.exposedRect = rect

        # TODO bunlar niye koyuldu?
        option.exposedRect = self._rect
        option.rect = self._rect.toRect()

        # painter.setClipRect(option.exposedRect)
        super(Text, self).paint(painter, option, widget)

        # self.doc.drawContents(painter, self._rect)

        # # # # # # debug start - pos() # # # # #
        # p = self.pos()
        # s = self.scenePos()
        # painter.drawText(self._rect,
        #                  "{0:.2f},  {1:.2f} pos \n{2:.2f},  {3:.2f} spos".format(p.x(), p.y(), s.x(), s.y()))
        # painter.setPen(QPen(Qt.green, 12))
        # painter.drawPoint(self.mapFromScene(self.sceneBoundingRect().center()))
        # # # t = self.transformOriginPoint()
        # # # painter.drawRect(t.x()-12, t.y()-12,24,24)
        # mapped = self.mapToScene(self._rect.topLeft())
        # painter.drawText(self._rect.x(), self._rect.y(), "{0:.2f}  {1:.2f} map".format(mapped.x(), mapped.y()))
        # painter.drawEllipse(self.scenePos(), 10, 10)
        # painter.setPen(Qt.blue)
        # painter.drawEllipse(self.mapFromScene(self.pos()), 10, 10)
        # r = self.textItem.boundingRect()
        # r = self.mapRectFromItem(self.textItem, r)
        # painter.drawRect(r)
        # painter.drawText(self._rect.center(), "{0:f}  {1:f}".format(self.sceneWidth(), self.sceneHeight()))
        # painter.setPen(QPen(Qt.red,17))
        # painter.drawPoint(self._rect.center())
        # painter.setPen(QPen(Qt.green,12))
        # painter.drawPoint(self.boundingRect().center())
        # painter.setPen(QPen(Qt.blue,8))
        # painter.drawPoint(self.sceneBoundingRect().center())
        # painter.drawRect(self.sceneBoundingRect())
        # # # # # # debug end - pos() # # # # #



# -*- coding: utf-8 -*-
# .

__project_name__ = 'defter'
__date__ = '12/03/24'
__author__ = 'E. Y.'

from PySide6.QtWidgets import (QDialog, QVBoxLayout, QHBoxLayout, QPushButton, QApplication,
                               QLabel, QCheckBox, QTimeEdit, QMessageBox)
from PySide6.QtCore import Qt, Slot, QSettings, Signal, QTime
from PySide6.QtGui import QColor
from . import shared
from .spinBoxlar import SpinBox


########################################################################
class VideoyuResmeCevirPenceresi(QDialog):
    donusturTiklandi = Signal(int, int, bool, int, int, int, int, int, int, int, int)
    iptalTiklandi = Signal()

    # ---------------------------------------------------------------------
    def __init__(self, boyut_point, video_sure, altyazi_var_mi, altyazi_sayisi, parent=None):
        super(VideoyuResmeCevirPenceresi, self).__init__(parent)

        self.setAttribute(Qt.WidgetAttribute.WA_DeleteOnClose)
        self.setWindowFlags(Qt.WindowType.WindowStaysOnTopHint)
        self.setWindowTitle(self.tr("Defter: Video to Image Settings"))
        self.setModal(True)
        layout = QVBoxLayout(self)

        if boyut_point:
            genislik = boyut_point.width()
            yukseklik = boyut_point.height()
        else:
            genislik = 800
            yukseklik = 600

        self.oran = genislik / yukseklik

        # lblOraniKoru = QLabel(self.tr("Keep aspect ratio"), self)
        self.oraniKoruCBox = QCheckBox(self)
        self.oraniKoruCBox.setText(self.tr("Keep aspect ratio"))
        self.oraniKoruCBox.setChecked(True)

        lblGenislik = QLabel(self.tr("Width"), self)
        self.genislikSBox = SpinBox(self)
        self.genislikSBox.setSuffix(" px")
        self.genislikSBox.setMinimum(1)
        self.genislikSBox.setMaximum(9999)
        self.genislikSBox.setValue(genislik)
        self.genislikSBox.valueChangedFromSpinBoxGuiNotBySetValue.connect(self.act_genislik_degisti)
        layGenislik = QHBoxLayout()
        layGenislik.addWidget(lblGenislik)
        layGenislik.addWidget(self.genislikSBox)

        lblYukseklik = QLabel(self.tr("Height"), self)
        self.yukseklikSBox = SpinBox(self)
        self.yukseklikSBox.setSuffix(" px")
        self.yukseklikSBox.setMinimum(1)
        self.yukseklikSBox.setMaximum(9999)
        self.yukseklikSBox.setValue(yukseklik)
        self.yukseklikSBox.valueChangedFromSpinBoxGuiNotBySetValue.connect(self.act_yukseklik_degisti)

        layYukseklik = QHBoxLayout()
        layYukseklik.addWidget(lblYukseklik)
        layYukseklik.addWidget(self.yukseklikSBox)

        self.orjinalBoyutuKoru = QCheckBox(self)
        self.orjinalBoyutuKoru.setText(self.tr("Keep original size"))
        self.orjinalBoyutuKoru.setToolTip(self.tr("If selected, images are saved at the original size but added to the scene at the dimensions entered above."))
        self.orjinalBoyutuKoru.setChecked(False)

        lblResimArasiBosluk = QLabel(self.tr("Gap bw. images"), self)
        self.resimArasiDikeyBoslukSBox = SpinBox(self)
        self.resimArasiDikeyBoslukSBox.setMinimum(0)
        self.resimArasiDikeyBoslukSBox.setMaximum(1000)
        self.resimArasiDikeyBoslukSBox.setValue(10)
        layResimArasiBosluk = QHBoxLayout()
        layResimArasiBosluk.addWidget(lblResimArasiBosluk)
        layResimArasiBosluk.addWidget(self.resimArasiDikeyBoslukSBox)

        lblBaslangic = QLabel(self.tr("From"), self)
        self.baslangicTE = QTimeEdit(self)
        self.baslangicTE.setDisplayFormat("HH:mm:ss:zzz")
        self.baslangicTE.setMaximumTime(video_sure)
        # self.baslangicTE.setMinimum(1)
        # self.baslangicTE.setMaximum(altyazi_sayisi)
        self.baslangicTE.timeChanged.connect(self.act_resim_sayisi_hesapla)
        layBaslangic = QHBoxLayout()
        layBaslangic.addWidget(lblBaslangic)
        layBaslangic.addWidget(self.baslangicTE)

        lblBitis = QLabel(self.tr("To"), self)
        self.bitisTE = QTimeEdit(self)
        self.bitisTE.setDisplayFormat("HH:mm:ss:zzz")
        self.bitisTE.setTime(video_sure)
        self.bitisTE.setMaximumTime(video_sure)
        # self.bitisTE.setMinimum(1)
        # self.bitisTE.setMaximum(altyazi_sayisi)
        # self.bitisTE.setValue(altyazi_sayisi)
        self.bitisTE.timeChanged.connect(self.act_resim_sayisi_hesapla)
        layBitis = QHBoxLayout()
        layBitis.addWidget(lblBitis)
        layBitis.addWidget(self.bitisTE)

        lblZamanAraligi = QLabel(self.tr("Interval"), self)
        self.zamanAraligiTE = QTimeEdit(self)
        self.zamanAraligiTE.setDisplayFormat("HH:mm:ss:zzz")
        self.zamanAraligiTE.setMinimumTime(QTime(0, 0, 0, 1))
        self.zamanAraligiTE.setMaximumTime(video_sure)
        self.zamanAraligiTE.timeChanged.connect(self.act_resim_sayisi_hesapla)
        layZamanAraligi = QHBoxLayout()
        layZamanAraligi.addWidget(lblZamanAraligi)
        layZamanAraligi.addWidget(self.zamanAraligiTE)

        self.lblToplamResimSayisi = QLabel(self.tr("0 Image"), self)
        self.lblToplamResimSayisi.setAlignment(Qt.AlignmentFlag.AlignCenter)
        self.lblToplamResimSayisi.setAutoFillBackground(True)
        p = self.lblToplamResimSayisi.palette()
        p.setColor(self.lblToplamResimSayisi.foregroundRole(), QColor(255, 255, 255))
        p.setColor(self.lblToplamResimSayisi.backgroundRole(), QColor(140, 160, 170))
        self.lblToplamResimSayisi.setPalette(p)
        self.act_resim_sayisi_hesapla()

        self.altyaziCBox = QCheckBox(self.tr("With subtitles"), self)
        self.altyaziCBox.stateChanged.connect(self.act_altyaziCBox_degisti)
        layAltyazi = QHBoxLayout()
        layAltyazi.addWidget(self.altyaziCBox)

        lblOncekiAltyazi = QLabel(self.tr("Merge with prev. subtitle"), self)
        self.oncekiAltyaziSBox = SpinBox(self)
        self.oncekiAltyaziSBox.setValue(0)
        self.oncekiAltyaziSBox.setMinimum(0)
        self.oncekiAltyaziSBox.setMaximum(altyazi_sayisi)
        layOncekiAltyazi = QHBoxLayout()
        layOncekiAltyazi.addWidget(lblOncekiAltyazi)
        layOncekiAltyazi.addWidget(self.oncekiAltyaziSBox)

        lblSonrakiAltyazi = QLabel(self.tr("Merge with next subtitle"), self)
        self.sonrakiAltyaziSBox = SpinBox(self)
        self.sonrakiAltyaziSBox.setValue(0)
        self.sonrakiAltyaziSBox.setMinimum(0)
        self.sonrakiAltyaziSBox.setMaximum(altyazi_sayisi)
        laySonrakiAltyazi = QHBoxLayout()
        laySonrakiAltyazi.addWidget(lblSonrakiAltyazi)
        laySonrakiAltyazi.addWidget(self.sonrakiAltyaziSBox)

        if not altyazi_var_mi:
            self.altyaziCBox.setText(self.tr("No subtitles were added to the video"))
            self.altyaziCBox.setToolTip(self.tr("You can add subtitles to the video and try again."))
            self.altyaziCBox.setChecked(False)
            self.altyaziCBox.setDisabled(True)
            self.oncekiAltyaziSBox.setDisabled(True)
            self.sonrakiAltyaziSBox.setDisabled(True)
        else:
            self.altyaziCBox.setChecked(True)

        donusturBtn = QPushButton(self.tr("&Extract"), self)
        donusturBtn.clicked.connect(self.act_donustur)
        self.kapatBtn = QPushButton(self.tr("&Close"), self)
        self.kapatBtn.setDefault(True)
        self.kapatBtn.clicked.connect(self.kapat_yada_iptal_et)
        layBtn = QHBoxLayout()
        layBtn.addWidget(donusturBtn)
        layBtn.addWidget(self.kapatBtn)

        layout.addWidget(self.oraniKoruCBox)
        layout.addLayout(layGenislik)
        layout.addLayout(layYukseklik)
        layout.addLayout(layResimArasiBosluk)
        layout.addWidget(self.orjinalBoyutuKoru)
        layout.addLayout(layBaslangic)
        layout.addLayout(layBitis)
        layout.addLayout(layZamanAraligi)
        layout.addWidget(self.lblToplamResimSayisi)
        layout.addLayout(layAltyazi)
        layout.addLayout(layOncekiAltyazi)
        layout.addLayout(laySonrakiAltyazi)
        layout.addLayout(layBtn)

        self.olustur_ayarlar()
        self.oku_ayarlar()
        # commandDialog.accepted.connect()

    # ---------------------------------------------------------------------
    @Slot()
    def kapat_yada_iptal_et(self):

        if self.kapatBtn.text == "&Close":
            self.reject()
        else:
            self.iptalTiklandi.emit()
            self.reject()

    # ---------------------------------------------------------------------
    @Slot(QTime)
    def act_resim_sayisi_hesapla(self, lazim_degil=None):

        self.resimSayisi = ((self.bitisTE.time().msecsSinceStartOfDay()
                             - self.baslangicTE.time().msecsSinceStartOfDay())
                            // self.zamanAraligiTE.time().msecsSinceStartOfDay()) + 1
        self.lblToplamResimSayisi.setText(self.tr("%n image(s)", "", self.resimSayisi))

    # ---------------------------------------------------------------------
    @Slot(int)
    def act_altyaziCBox_degisti(self, deger):
        self.oncekiAltyaziSBox.setDisabled(not deger)
        self.sonrakiAltyaziSBox.setDisabled(not deger)

    # ---------------------------------------------------------------------
    @Slot(int)
    def act_genislik_degisti(self, deger):
        if self.oraniKoruCBox.isChecked():
            self.yukseklikSBox.setValue(self.genislikSBox.value() / self.oran)

    # ---------------------------------------------------------------------
    @Slot(int)
    def act_yukseklik_degisti(self, deger):
        if self.oraniKoruCBox.isChecked():
            self.genislikSBox.setValue(self.yukseklikSBox.value() * self.oran)

    # ---------------------------------------------------------------------
    @Slot()
    def act_donustur(self):

        # ilk_ms = ((ilk.hour() * 3600 + ilk.minute()*60 +ilk.second()*1000) * 1000 ) + ilk.msec()
        # son_ms = ((son.hour() * 3600 + son.minute()*60 +son.second()*1000) * 1000 ) + son.msec()
        # zaman_araligi_ms = ((zaman_araligi.hour() * 3600 + zaman_araligi.minute()*60 +zaman_araligi.second()*1000) * 1000 ) + zaman_araligi.msec()

        if self.resimSayisi > 100:
            msgBox = QMessageBox(self)
            msgBox.setWindowTitle(self.tr(f"Defter: There are {self.resimSayisi} images to extract!"))
            msgBox.setIcon(QMessageBox.Icon.Warning)
            msgBox.setText(self.tr(f"Do you want to extract {self.resimSayisi} images?"))
            msgBox.setInformativeText(self.tr("This may take time!"))

            extractButton = msgBox.addButton(self.tr("&Extract"), QMessageBox.ButtonRole.AcceptRole)
            cancelButton = msgBox.addButton(self.tr("&Cancel"), QMessageBox.ButtonRole.RejectRole)
            # msgBox.addButton(QMessageBox.Cancel)
            msgBox.setDefaultButton(extractButton)
            msgBox.setEscapeButton(cancelButton)

            msgBox.exec()
            if msgBox.clickedButton() == extractButton:
                devam = True
            else:
                devam = False
        else:
            devam = True

        if devam:
            # self.kapatBtn.setText(self.tr("&Cancel"))
            self.donusturTiklandi.emit(self.genislikSBox.value(),
                                       self.yukseklikSBox.value(),
                                       self.orjinalBoyutuKoru.isChecked(),
                                       self.baslangicTE.time().msecsSinceStartOfDay(),
                                       self.bitisTE.time().msecsSinceStartOfDay(),
                                       self.zamanAraligiTE.time().msecsSinceStartOfDay(),
                                       self.resimArasiDikeyBoslukSBox.value(),
                                       self.resimSayisi,
                                       self.altyaziCBox.isChecked(),
                                       self.oncekiAltyaziSBox.value(),
                                       self.sonrakiAltyaziSBox.value()
                                       )

            # self.lblToplamResimSayisi.setText(self.tr("Please wait..."))

            self.accept()

    # ---------------------------------------------------------------------
    def closeEvent(self, event):
        self.yaz_ayarlar()
        event.accept()

    # ---------------------------------------------------------------------
    def olustur_ayarlar(self):
        QApplication.setOrganizationName(shared.DEFTER_ORG_NAME)
        QApplication.setOrganizationDomain(shared.DEFTER_ORG_DOMAIN)
        QApplication.setApplicationName(shared.DEFTER_APP_NAME)
        self.settings = QSettings(shared.DEFTER_AYARLAR_DOSYA_ADRES, QSettings.Format.IniFormat)
        # self.settings.clear()

    # ---------------------------------------------------------------------
    def oku_ayarlar(self):
        self.settings.beginGroup("VideoyuResmeCevirPenceresi")
        if self.settings.contains("CWinGeometry"):
            self.restoreGeometry(self.settings.value("CWinGeometry"))
        self.settings.endGroup()

    # ---------------------------------------------------------------------
    def yaz_ayarlar(self):
        self.settings.beginGroup("VideoyuResmeCevirPenceresi")
        self.settings.setValue("CWinGeometry", self.saveGeometry())
        self.settings.endGroup()
